/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */

#ifndef SCRATCH_EVENTTRACE_H
#define SCRATCH_EVENTTRACE_H

#include "ns3/wifi-module.h"
#include "ns3/wifi-mac-header.h"

void
PhyTxTrace (std::string context, Ptr<const Packet> packet, Ptr<const WifiPhyTag> phytag)
{
  std::cout << "PHYTX mode=" << phytag->GetWifiMode() << " " << *packet << std::endl;

  Ptr<Packet> packetcopy = packet->Copy();

  WifiMacHeader hdr;
  packetcopy->RemoveHeader(hdr);

  WifiMacTrailer trl;
  packetcopy->RemoveTrailer(trl);

  std::cout << "EVENT event=wifipacket context=" << context
            << " start=" << Simulator::Now()
            << " duration=" << phytag->GetDuration()
            << " action=tx"
            << " uid=" << packet->GetUid()
            << " type=" << hdr.GetTypeString()
            << " destination=" << hdr.GetAddr1()
            << " size=" << packetcopy->GetSize()
            << " phytag={" << *phytag << "}"
            << std::endl;
}
void
PhyRxStartTrace (std::string context, Ptr<const Packet> packet, Ptr<const WifiPhyTag> phytag)
{
  Ptr<Packet> packetcopy = packet->Copy();

  WifiMacHeader hdr;
  packetcopy->RemoveHeader(hdr);

  WifiMacTrailer trl;
  packetcopy->RemoveTrailer(trl);

  std::cout << "EVENT event=wifipacket context=" << context
            << " start=" << Simulator::Now()
            << " duration=" << phytag->GetDuration()
            << " action=rxstart"
            << " uid=" << packet->GetUid()
            << " type=" << hdr.GetTypeString()
            << " destination=" << hdr.GetAddr1()
            << " size=" << packetcopy->GetSize()
            << " phytag={" << *phytag << "}"
            << std::endl;
}
void
PhyRxOkTrace (std::string context, Ptr<const Packet> packet, Ptr<const WifiPhyTag> phytag)
{
  std::cout << "PHYRXOK mode=" << phytag->GetWifiMode() << " snr=" << phytag->GetRxSnr() << " " << *packet << std::endl;

  Ptr<Packet> packetcopy = packet->Copy();

  WifiMacHeader hdr;
  packetcopy->RemoveHeader(hdr);

  WifiMacTrailer trl;
  packetcopy->RemoveTrailer(trl);

  std::cout << "EVENT event=wifipacket context=" << context
            << " start=" << Simulator::Now()
            << " duration=" << phytag->GetDuration()
            << " action=rx"
            << " uid=" << packet->GetUid()
            << " type=" << hdr.GetTypeString()
            << " destination=" << hdr.GetAddr1()
            << " size=" << packetcopy->GetSize()
            << " phytag={" << *phytag << "}"
            << std::endl;
}
void
PhyRxErrorTrace (std::string context, Ptr<const Packet> packet, Ptr<const WifiPhyTag> phytag, WifiPhy::RxErrorReason rxErrorReason)
{
  std::cout << "PHYRXERROR snr=" << phytag->GetRxSnr()
            << " reason=" << WifiPhy::RxErrorReasonToString(rxErrorReason)
            << " " << *packet << std::endl;

  Ptr<Packet> packetcopy = packet->Copy();

  WifiMacHeader hdr;
  packetcopy->RemoveHeader(hdr);

  WifiMacTrailer trl;
  packetcopy->RemoveTrailer(trl);

  std::cout << "EVENT event=wifipacket context=" << context
            << " start=" << Simulator::Now()
            << " duration=" << phytag->GetDuration()
            << " action=rxerror"
            << " uid=" << packet->GetUid()
            << " type=" << hdr.GetTypeString()
            << " destination=" << hdr.GetAddr1()
            << " size=" << packetcopy->GetSize()
            << " phytag={" << *phytag << "}"
            << " reason=" << WifiPhy::RxErrorReasonToString(rxErrorReason)
            << std::endl;
}
void
PhyStateTrace (std::string context, Time start, Time duration, enum WifiPhy::State state)
{
  std::cout << "EVENT event=wifistate context=" << context
            << " state=" << WifiPhy::StateToString(state)
            << " time=" << start
            << " duration=" << duration
            << std::endl;
}

void
ConnectEventTrace()
{
  Config::Connect ("/NodeList/*/DeviceList/*/Phy/State/RxStart", MakeCallback (&PhyRxStartTrace));
  Config::Connect ("/NodeList/*/DeviceList/*/Phy/State/RxOk", MakeCallback (&PhyRxOkTrace));
  Config::Connect ("/NodeList/*/DeviceList/*/Phy/State/RxError", MakeCallback (&PhyRxErrorTrace));
  Config::Connect ("/NodeList/*/DeviceList/*/Phy/State/Tx", MakeCallback (&PhyTxTrace));
  Config::Connect ("/NodeList/*/DeviceList/*/Phy/State/State", MakeCallback (&PhyStateTrace));
}

#endif // SCRATCH_EVENTTRACE_H
