/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2006,2007 INRIA, 2009 Timo Bingmann
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as 
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Based on yans-wifi-channel.h by
 * Mathieu Lacage <mathieu.lacage@sophia.inria.fr>
 * Minor Modifications: Timo Bingmann <timo.bingmann@student.kit.edu>
 */
#ifndef NS2EXT_WIFI_CHANNEL_H
#define NS2EXT_WIFI_CHANNEL_H

#include <vector>
#include <stdint.h>
#include "ns3/packet.h"
#include "wifi-channel.h"
#include "wifi-mode.h"
#include "wifi-preamble.h"

namespace ns3 {

class NetDevice;
class PropagationLossModel;
class PropagationDelayModel;
class Ns2ExtWifiPhy;
class WifiPhyTag;

/**
 * \brief A Ns2Ext wifi channel
 *
 * This class is a clone of ns3::YansWifiChannel. The cloning is necessary
 * because the class contains a std::vector of attached WifiPhys.
 */
class Ns2ExtWifiChannel : public WifiChannel
{
public:
  static TypeId GetTypeId (void);

  Ns2ExtWifiChannel ();
  virtual ~Ns2ExtWifiChannel ();

  // inherited from Channel.
  virtual uint32_t GetNDevices (void) const;
  virtual Ptr<NetDevice> GetDevice (uint32_t i) const;

  void Add (Ptr<Ns2ExtWifiPhy> phy);

  /**
   * \param loss the new propagation loss model.
   */
  void SetPropagationLossModel (Ptr<PropagationLossModel> loss);
  /**
   * \param delay the new propagation delay model.
   */
  void SetPropagationDelayModel (Ptr<PropagationDelayModel> delay);

  /**
   * \param sender the device from which the packet is originating.
   * \param packet the packet to send
   * \param tag the WifiPhy tag associated to the packet
   *
   * This method should not be invoked by normal users. It is 
   * currently invoked only from WifiPhy::Send. YansWifiChannel 
   * delivers packets only between PHYs with the same m_channelNumber,
   * e.g. PHYs that are operating on the same channel.
   */
  void Send (Ptr<Ns2ExtWifiPhy> sender, Ptr<const Packet> packet, Ptr<const WifiPhyTag> tag) const;

private:
  typedef std::vector<Ptr<Ns2ExtWifiPhy> > PhyList;
  void Receive (uint32_t i, Ptr<Packet> packet, Ptr<WifiPhyTag> tag,
                double rxPowerDbm) const;

  PhyList m_phyList;
  Ptr<PropagationLossModel> m_loss;
  Ptr<PropagationDelayModel> m_delay;
};

} // namespace ns3


#endif /* NS2EXT_WIFI_CHANNEL_H */
